#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
    @author: Gabriele Di Bari
    @todo: collision and maps
    @version: 0.9
"""
import cocos
import pyglet
import pymunk
import math
from cocos.director import director
from cocos.text import *
from bullet import *

def dist(v1,v2):
    vdif=(v1[0]-v2[0],v1[1]-v2[1])
    return math.sqrt(vdif[0]*vdif[0]+vdif[1]*vdif[1])

def diff(v1,v2):
    return (v1[0]-v2[0],v1[1]-v2[1])

def length(v1):
    return math.sqrt(v1[0]*v1[0]+v1[1]*v1[1])

def normalize(v1):    
    fval=length(v1)
    return (v1[0]/fval,v1[1]/fval)

def dot(v1,v2):
    return v1[0]*v2[0]+v1[1]*v2[1]

def vstring(v1):
    return "(",v1[0],",",v1[1],")"

class KeyStatus:
    
    def __init__(self):
        self.up= False
        self.down= False
        self.left= False        
        self.right= False
        self.reload=False
        
class MouseStatus:
    
    def __init__(self):        
        self.x=0
        self.y=0
        self.press= False

class Weapon:
    
    def __init__(self,loaderSize,loaderMax,loaderCountStart,bulletCountStart):
        """
            loaderSize=loader size
            loaderMax=max loader number
            loaderCountStart=star loader number
            bulletCountStart=star bullet in last loader
        """
        self.loaderSize=loaderSize
        self.loaderMax=loaderMax
        self.loaderCount=loaderCountStart
        self.bulletCount=(loaderCountStart*loaderSize+bulletCountStart)%(loaderMax*loaderSize)
        self.bulletLoader=self.bulletCount%self.loaderSize
        
    def Fire(self):
        """
        if bullet in loader
            Decrement bullet count, and return true
        else
            return false
        """
        if self.bulletLoader!=0:
            self.bulletCount-=1
            self.bulletLoader=self.bulletCount%self.loaderSize
            return True
        else:
            return False
    
    def Reload(self):
        """
         Reload loader 
        """
        if self.loaderCount>0:
            self.bulletCount-=self.BulletInLoader()
            self.bulletLoader=self.loaderSize
            self.loaderCount-=1
            return True
        else:
            return False
    
    def AddLoder(self):
        """
         increment count loader
        """
        if self.loaderMax>self.loaderCount:
            self.loaderCount+=1
            return True
        else:
            return False
    
    def BulletInLoader(self):
        """
         return bullet number (in loader)
        """
        return self.bulletLoader
    
    def LoadersWithBullet(self):
        """
            Number loader (with bullet)
        """
        return self.loaderCount

class Player:
    
    def __init__(self,colorLayer,chipmunkSpace,vars):
        """
            colorLayer= cocos layer
            chipmunkSpace=chipmunk space 
            vars={
                      "position":(?,?),            #world player position
                      "image_player":"Player.png", #player image
                      "image_bullet":"bullet.png", #bullet image
                      "image_weapon":              #weapon image
                      "distance_weapon":           #distance weapon (from player)
                      "weapon": Weapon(bullet in loader, max loader, loader start, bullet in loader start)
                      "effects": Audio Effects,
                      "range_body": value body shape chipmunk
                      "scale_body": value scale body cocos,
                      "scale_weapon" value scale weapon cocos
                      }
        """
        self.keystatus=KeyStatus()
        self.mouse=MouseStatus()    
        self.weapon=vars["weapon"]
        self.cpSpace=chipmunkSpace
        self.colorLayer=colorLayer
        self.position_start=vars["position"]
        self.image_sprite=vars["image_player"]
        self.image_bullet=vars["image_bullet"]
        self.image_weapon=vars["image_weapon"]
        self.distance_weapon=vars["distance_weapon"]
        self.effects = vars['effects']
        #Graphycs
        #cocos body
        self.ccbody=cocos.sprite.Sprite(self.image_sprite, scale=vars["scale_body"])
        self.ccweapon=cocos.sprite.Sprite(self.image_weapon, scale=vars["scale_weapon"])
        #add into cocos layer
        colorLayer.add(self.ccbody, z = 8)
        colorLayer.add(self.ccweapon, z = 7)
        
        #Physics
        #physics body, diable rotation
        self.cpbody = pymunk.Body(10,pymunk.inf)  
        #add geometry (ball)
        self.cpGeometry = pymunk.Circle( self.cpbody,vars["range_body"],(0,0))
        self.cpGeometry._set_collision_type(4)
        self.cpGeometry._set_layers(3)
        #add into chipmunk scene
        chipmunkSpace.add(self.cpbody,self.cpGeometry)
        #add list bullet
        self.bullets=[]
        #set start  position    
        self.cpbody.position=self.position_start
        self.cpbody.player = self
        self.ccbody.position=self.cpbody.position  
        #pointer count and life count
        self.point=0
        self.life=100.0
    
    def __updateweapon(self):
        pcos=math.cos(math.radians(self.ccweapon.rotation))
        psin=-math.sin(math.radians(self.ccweapon.rotation))
        bl_x= pcos*self.distance_weapon[0]-psin*self.distance_weapon[1]+self.ccbody.position[0]
        bl_y= psin*self.distance_weapon[0]+pcos*self.distance_weapon[1]+self.ccbody.position[1]
        self.ccweapon.position=(bl_x,bl_y)
        self.ccweapon.rotation=self.ccbody.rotation
        
    
    def __upadatabullets(self):
        for b in self.bullets:
            if not b.update() :
            	self.bullets.remove(b)
                
      
    def Upadate(self):  
        """
         Update state player
        """
        self.cpbody.velocity=(0,0)
        if self.mouse.press:
            #print " Proiettili :"+str(self.weapon.bulletCount)       
            #print "Caricatori:"+str(self.weapon.LoadersWithBullet())+" Proiettili nel caricatore:"+str(self.weapon.BulletInLoader())
            if self.weapon.Fire():
                pointvec=diff(self.ccweapon.position,self.ccbody.position)
                argsBullet = {
                        'img' : self.image_bullet, 
                        'layer' : self.colorLayer,
                        'depth' : 6,
                        'space' : self.cpSpace,
                        'origin' : self.ccweapon.position,
                        'end' : (self.mouse.x+pointvec[0],self.mouse.y+pointvec[1]),
                        'vel' : 1000,
                        'player' : self
                        }
                bulletPh=Bullet(argsBullet)
                self.bullets.append(bulletPh)
                self.effects.play('shot')
            self.mouse.press=False
        
        if self.keystatus.reload:
            self.weapon.Reload()
            self.keystatus.reload=False
            
                
        if self.keystatus.up:
            self.cpbody.velocity[1]+=100
           
        if self.keystatus.down:            
            self.cpbody.velocity[1]-=100
           
        if self.keystatus.left:     
            self.cpbody.velocity[0]-=100
           
        if self.keystatus.right:     
            self.cpbody.velocity[0]+=100  
        
        #get dir
        rot_dir=(self.cpbody.position.x-self.mouse.x,self.cpbody.position.y-self.mouse.y)    
        self.ccbody.rotation=math.degrees(-math.atan2(rot_dir[1],rot_dir[0]))
        #update weapon image
        self.__updateweapon()
        #updata cocos position
        self.ccbody.position=self.cpbody.position  
        #updata bullets
        self.__upadatabullets()
        
    def MouseMotion (self, x, y, dx, dy):
        """
            Set mouse input
        """
        virtual_xy=director.get_virtual_coordinates(x,y)
        self.mouse.x=virtual_xy[0]
        self.mouse.y=virtual_xy[1]
        
    def MouseDown (self, x, y, buttons, modifiers):
        """
            Set mouse input
        """
        virtual_xy=director.get_virtual_coordinates(x,y)
        self.mouse.x=virtual_xy[0]
        self.mouse.y=virtual_xy[1]  
        if buttons==1:
            self.mouse.press= True
    
    def MouseUp (self, x, y, buttons, modifiers):
        """
            Set mouse input
        """
        virtual_xy=director.get_virtual_coordinates(x,y)
        self.mouse.x=virtual_xy[0]
        self.mouse.y=virtual_xy[1]
        if buttons==1:
            self.mouse.press= False
        
    def KeyUp(self,key):
        """
            Set keyboard input
        """
        
        if pyglet.window.key.symbol_string(key) == 'UP' or pyglet.window.key.symbol_string(key) == 'W':
            self.keystatus.up = False
            
        if pyglet.window.key.symbol_string(key) == 'DOWN' or pyglet.window.key.symbol_string(key) == 'S':
            self.keystatus.down = False
            
        if pyglet.window.key.symbol_string(key) == 'LEFT' or pyglet.window.key.symbol_string(key) == 'A':
            self.keystatus.left = False
            
        if pyglet.window.key.symbol_string(key) == 'RIGHT' or pyglet.window.key.symbol_string(key) == 'D':
            self.keystatus.right = False

        if key==pyglet.window.key.RCTRL or pyglet.window.key.symbol_string(key) == 'R':
            self.keystatus.reload = False
            
    def KeyDown(self,key):
        """
            Set keyboard input
        """
                
        if pyglet.window.key.symbol_string(key) == 'UP' or pyglet.window.key.symbol_string(key) == 'W':
            self.keystatus.up = True
            
        if pyglet.window.key.symbol_string(key) == 'DOWN' or pyglet.window.key.symbol_string(key) == 'S':
            self.keystatus.down = True
            
        if pyglet.window.key.symbol_string(key) == 'LEFT' or pyglet.window.key.symbol_string(key) == 'A':
            self.keystatus.left = True
            
        if pyglet.window.key.symbol_string(key) == 'RIGHT' or pyglet.window.key.symbol_string(key) == 'D':
            self.keystatus.right = True
        
        if key==pyglet.window.key.RCTRL or pyglet.window.key.symbol_string(key) == 'R':
            self.keystatus.reload = True


class DrawLife:
    
    def __init__(self,player,colorLayer,position):
        #save argument
        self.colorLayer=colorLayer
        self.player=player
        ######################################
        self.font = {}        
        self.font['font_size'] = 22
        self.font['color'] = (255,255,255,255)
        self.font['anchor_y'] = 'top'
        self.font['anchor_x'] = 'center'
        self.font['bold']=True
        ######################################
        self.label_position=position
        self.label=Label("Life "+str(int(self.player.life)), **self.font)
        self.label.position=self.label_position
        self.colorLayer.add(self.label, z = 15)
        
    def Update(self):
        self.label.element.text="Life "+str(int(self.player.life))
        if self.player.life>75:
            self.label.element.color=(255,255,255,255)
        elif self.player.life>25:
            self.label.element.color=(255,255,0,255)
        else:            
            self.label.element.color=(255,0,0,255)

class DrawPoint:
    
    def __init__(self,player,colorLayer,position):
        #save argument
        self.colorLayer=colorLayer
        self.player=player
        ######################################
        self.font = {}        
        self.font['font_size'] = 22
        self.font['color'] = (255,255,255,255)
        self.font['anchor_y'] = 'top'
        self.font['anchor_x'] = 'left'
        self.font['bold']=True
        ######################################
        self.label_position=(position[0]-50,position[1])
        self.label=Label("Score "+str(int(self.player.point)), **self.font)
        self.label.position=self.label_position
        self.colorLayer.add(self.label, z = 15)
        
    def Update(self):
        self.label.element.text="Score "+str(int(self.player.point))
            
class DrawWeaponLite:
    
    def __init__(self,weapon,colorLayer,valueZ,position,bullet_enable,bullet_disable):        
        ######################################
        self.position=position
        self.weapon=weapon
        self.font = {}        
        self.font['font_size'] = 33
        self.font['color'] = (255,255,255,255)
        self.font['anchor_y'] = 'top'
        self.font['anchor_x'] = 'center'
        self.font['bold']=True
        ######################################
        max_x_bullets=0        
        #create image bullet   
        self.bullet_disable=[]
        for i in range(0,self.weapon.loaderSize):
            tmp=cocos.sprite.Sprite(bullet_disable)
            rect = tmp.get_rect() 
            tmp.position=(position[0]+i*rect.width,position[1])
            tmp.visible=True
            colorLayer.add(tmp,valueZ)
            self.bullet_disable.append(tmp)
            max_x_bullets=position[0]+i*rect.width
        #create image bullet
        self.bullet_enable=[]        
        for i in range(0,self.weapon.loaderSize):
            tmp=cocos.sprite.Sprite(bullet_enable)
            rect = tmp.get_rect() 
            tmp.position=(position[0]+i*rect.width,position[1])
            tmp.visible=True
            colorLayer.add(tmp,valueZ+1)
            self.bullet_enable.append(tmp)
           
        #save layer
        self.colorLayer=colorLayer
        #label pos
        self.label_position=(max_x_bullets+55,position[1]+26)
        self.label=Label(str(self.weapon.LoadersWithBullet()*self.weapon.loaderSize), **self.font)
        self.colorLayer.add(self.label,valueZ+1)
        self.label.position=self.label_position

    
    def Update(self):
        self.label.element.text=str(self.weapon.LoadersWithBullet()*self.weapon.loaderSize)
        for i in range(0,self.weapon.loaderSize):
            if self.weapon.BulletInLoader()<=i:
                self.bullet_enable[i].visible=True
                self.bullet_disable[i].visible=False
            else:
                self.bullet_enable[i].visible=False
                self.bullet_disable[i].visible=True
                

            
class DrawWeaponClassic:
    
    def __init__(self,weapon,colorLayer,valueZ,position,bullet_enable,bullet_disable,loader_enable,loader_disable):        
        ######################################
        self.position=position
        self.weapon=weapon
        ######################################
        loader_h=0
        #create image loader    
        self.loader_disable=[]
        for i in range(0,self.weapon.loaderMax):
            tmp=cocos.sprite.Sprite(loader_disable)
            rect = tmp.get_rect() 
            loader_h=max(loader_h,rect.height) 
            tmp.position=(position[0]+i*rect.width,position[1])
            tmp.visible=True
            colorLayer.add(tmp,valueZ)
            self.loader_disable.append(tmp)
        #create image loader    
        self.loader_enable=[]
        for i in range(0,self.weapon.loaderMax):
            tmp=cocos.sprite.Sprite(loader_enable)
            rect = tmp.get_rect()
            loader_h=max(loader_h,rect.height)            
            tmp.position=(position[0]+i*rect.width,position[1])
            tmp.visible=True
            colorLayer.add(tmp,valueZ+1)
            self.loader_enable.append(tmp)
            
        ######################################        
        #create image bullet   
        self.bullet_disable=[]
        for i in range(0,self.weapon.loaderSize):
            tmp=cocos.sprite.Sprite(bullet_disable)
            rect = tmp.get_rect() 
            tmp.position=(position[0]+i*rect.width,position[1]+loader_h)
            tmp.visible=True
            colorLayer.add(tmp,valueZ)
            self.bullet_disable.append(tmp)
        #create image bullet
        self.bullet_enable=[]        
        for i in range(0,self.weapon.loaderSize):
            tmp=cocos.sprite.Sprite(bullet_enable)
            rect = tmp.get_rect() 
            tmp.position=(position[0]+i*rect.width,position[1]+loader_h)
            tmp.visible=True
            colorLayer.add(tmp,valueZ+1)
            self.bullet_enable.append(tmp)
            
        #save layer
        self.colorLayer=colorLayer
    
    def Update(self):
        
        for i in range(0,self.weapon.loaderSize):
            if self.weapon.BulletInLoader()<=i:
                self.bullet_enable[i].visible=True
                self.bullet_disable[i].visible=False
            else:
                self.bullet_enable[i].visible=False
                self.bullet_disable[i].visible=True
                
        for i in range(0,self.weapon.loaderMax):
            if self.weapon.LoadersWithBullet()<=i:
                self.loader_enable[i].visible=True
                self.loader_disable[i].visible=False
            else:
                self.loader_enable[i].visible=False
                self.bullet_disable[i].visible=True

            
